<?php
/*
 * Plugin Name:     Internal Link Scope
 * Plugin URL:      https://markpruett.com/internal-link-scope
 * Description:     Exports post header tags and internal links to CSV files.
 * Version:         1.0
 * Author:          Mark Pruett
 * Author URI:      https://markpruett.com
 * License:         GPL v2
 * License URI:     https://www.gnu.org/licenses/gpl-2.0.html
*/

function export_headers_to_csv() {
    // Query for all posts
    $args = array(
        'post_type' => 'post',
        'post_status' => 'publish',
        'posts_per_page' => -1,
    );

    $query = new WP_Query($args);

    $headers = array();

    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();

            // Extract headers from the post content
            preg_match_all('/<h([1-6])[^>]*>(.*?)<\/h\1>/is', apply_filters('the_content', get_the_content()), $matches);

            for ($i = 0; $i < count($matches[0]); $i++) {
                $header_tag = 'H' . $matches[1][$i];
                $header_text = strip_tags($matches[0][$i]);

                // Add the header to the array
                $headers[] = array(
                    'post_id' => get_the_ID(),
                    'post_title' => get_the_title(),
                    'post_url' => get_the_permalink(),
                    'header_tag' => $header_tag,
                    'header_text' => $header_text,
                );
            }
        }
        // Reset Post Data
        wp_reset_postdata();
    }

    // Send download headers
    header('Content-Type: text/csv; charset=utf-8');
    header('Content-Disposition: attachment; filename=headers.csv');

    // Create a file pointer connected to the output stream
    $output = fopen('php://output', 'w');

    // Output the column headings
    fputcsv($output, array('Post ID', 'Post Title', 'Post URL', 'Header Tag', 'Header Text'));

    // Output the headers
    foreach ($headers as $header) {
        fputcsv($output, $header);
    }

    // Close the output stream
    fclose($output);

    // End execution to avoid extra output
    die();
}

function export_internal_links_to_csv() {
    // Query for all posts
    $args = array(
        'post_type' => 'post',
        'post_status' => 'publish',
        'posts_per_page' => -1,
    );

    $query = new WP_Query($args);
    
    $links = array();

    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            
            // Extract links from the post content
            preg_match_all('/<a(.*?)href="([^"]+)"(.*?)>(.*?)<\/a>/i', apply_filters('the_content', get_the_content()), $matches);
            
            for ($i = 0; $i < count($matches[2]); $i++) {
                $href = $matches[2][$i];
                $text = $matches[4][$i];

                // Check if the link is internal
                if (strpos($href, get_site_url()) === 0) {
                    // Parse URLs to remove potential URL parameters or anchor tags
                    $parsed_current_url = parse_url(get_the_permalink());
                    $parsed_link_url = parse_url($href);

                    // Check if the URLs point to the same page
                    if ($parsed_current_url['path'] != $parsed_link_url['path']) {
                        // Get the post ID of the linked post
                        $post_id = url_to_postid($href);

                        // Add the link to the array
                        $links[] = array(
                            'from_post_id' => get_the_ID(),
                            'from_post_title' => get_the_title(),
                            'to_post_id' => $post_id,
                            'to_post_title' => get_the_title($post_id),
                            'link_text' => $text,
                        );
                    }
                }
            }
        }
        // Reset Post Data
        wp_reset_postdata();
    }

    // Send download headers
    header('Content-Type: text/csv; charset=utf-8');
    header('Content-Disposition: attachment; filename=internal_links.csv');

    // Create a file pointer connected to the output stream
    $output = fopen('php://output', 'w');

    // Output the column headings
    fputcsv($output, array('From Post ID', 'From Post Title', 'To Post ID', 'To Post Title', 'Link Text'));

    // Output the links
    foreach ($links as $link) {
        fputcsv($output, $link);
    }

    // Close the output stream
    fclose($output);

    // End execution to avoid extra output
    die();
}

function export_internal_link_counts_to_csv() {
    // Query for all posts
    $args = array(
        'post_type' => 'post',
        'post_status' => 'publish',
        'posts_per_page' => -1,
    );

    $query = new WP_Query($args);

    // Prepare an array to store the link counts
    $link_counts = array();

    // Initialize the link counts for all posts
    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            $link_counts[get_the_ID()] = array('count' => 0, 'url' => get_the_permalink());
        }
        // Reset Post Data
        wp_reset_postdata();
    }

    // Loop through the posts again and count the internal links
    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            
            // Extract links from the post content
            preg_match_all('/<a(.*?)href="([^"]+)"(.*?)>/i', apply_filters('the_content', get_the_content()), $matches);
            
            foreach ($matches[2] as $href) {
                // Check if the link is internal
                if (strpos($href, get_site_url()) === 0) {
                    // Parse URLs to remove potential URL parameters or anchor tags
                    $parsed_current_url = parse_url(get_the_permalink());
                    $parsed_link_url = parse_url($href);

                    // Check if the URLs point to the same page
                    if ($parsed_current_url['path'] != $parsed_link_url['path']) {
                        // Get the post ID of the linked post
                        $post_id = url_to_postid($href);

                        // Increment the link count for the linked post
                        if (isset($link_counts[$post_id])) {
                            $link_counts[$post_id]['count']++;
                        }
                    }
                }
            }
        }
        // Reset Post Data
        wp_reset_postdata();
    }

    // Send download headers
    header('Content-Type: text/csv; charset=utf-8');
    header('Content-Disposition: attachment; filename=internal_link_counts.csv');

    // Create a file pointer connected to the output stream
    $output = fopen('php://output', 'w');

    // Output the column headings
    fputcsv($output, array('Post ID', 'Post Title', 'Post URL', 'Link Count'));

    // Output the link counts
    foreach ($link_counts as $post_id => $data) {
        fputcsv($output, array($post_id, get_the_title($post_id), $data['url'], $data['count']));
    }

    // Close the output stream
    fclose($output);

    // End execution to avoid extra output
    die();
}


// Hook for 'admin_post_' . $action
add_action('admin_post_export_headers', 'export_headers_to_csv');
add_action('admin_post_export_links', 'export_internal_links_to_csv');
add_action('admin_post_export_link_counts', 'export_internal_link_counts_to_csv');

// Add submenu page in the Dashboard
function add_submenu() {
    add_submenu_page(
        'tools.php',
        'Internal Link Scope 🔬',
        'Internal Link Scope 🔬',
        'manage_options',
        'export_post_data',
        'export_post_data_callback'
    );
}
add_action('admin_menu', 'add_submenu');

function export_post_data_callback() {
    // security check
    if (!current_user_can('manage_options')) {
        return;
    }
    
    ?>
        <div class="wrap" style="font-family: Arial, sans-serif; padding: 20px; color: #333;">

        <h1 style="margin-bottom: 30px; font-size: 32px; color: #444;"><?php echo esc_html(get_admin_page_title()); ?></h1>
        
        <div style="margin-bottom: 20px; font-size: 18px; line-height: 1.6;">
            This is particularly useful for auditing your site's internal linking strategy, ensuring your content is well-interconnected and user navigation is optimized.
        </div>
        
        <div style="margin-bottom: 30px;">
            <a href="<?php echo esc_url(admin_url('/admin-post.php?action=export_links')); ?>" class="page-title-action" style="display: inline-block; text-decoration: none; background-color: #0071a1; color: #fff; padding: 10px 15px; border-radius: 3px;">Export Internal Links to CSV</a>
        </div>
        
        <div style="margin-bottom: 20px; font-size: 18px; line-height: 1.6;">
            This function combs through all published posts, tallying the number of internal links leading to each post.
        </div>
        
        <div style="margin-bottom: 30px;">
            <a href="<?php echo esc_url(admin_url('/admin-post.php?action=export_link_counts')); ?>" class="page-title-action" style="display: inline-block; text-decoration: none; background-color: #0071a1; color: #fff; padding: 10px 15px; border-radius: 3px;">Export Internal Link Counts to CSV</a>
        </div>

        <div style="margin-bottom: 20px; font-size: 18px; line-height: 1.6;">
            A valuable tool for conducting in-depth SEO analysis, streamlining your content auditing process, or for gaining valuable insights into the structure and organization of your website's content.
        </div>

        <div style="margin-bottom: 60px;">
            <a href="<?php echo esc_url(admin_url('/admin-post.php?action=export_headers')); ?>" class="page-title-action" style="display: inline-block; text-decoration: none; background-color: #0071a1; color: #fff; padding: 10px 15px; border-radius: 3px;">Export Headers to CSV</a>
        </div>

        <div style="margin-bottom: 20px; font-size: 18px; line-height: 1.6;">
            Frase helps you write content that ranks better using keyword analysis on your competitors. (This is totally an affiliate link, but I use this product myself and recommend it)
        </div>

        <div style="margin-top: 30px; padding: 15px; background-color: #0071a1; color: #fff; text-align: center; border-radius: 3px;">
            <a href="https://www.frase.io/?via=mark71" style="font-size: 20px; font-weight: bold; text-decoration: none; color: #fff;">Boost your SEO Rankings with Frase!</a>
        </div>

        </div>
    <?php
}
?>
